from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.ogp_media import OgpMedia


T = TypeVar("T", bound="Ogp")


@_attrs_define
class Ogp:
    """OGPの情報

    Attributes:
        type (str):
        title (str):
        url (str):
        images (List['OgpMedia']):
        description (str):
        videos (List['OgpMedia']):
    """

    type: str
    title: str
    url: str
    images: List["OgpMedia"]
    description: str
    videos: List["OgpMedia"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type

        title = self.title

        url = self.url

        images = []
        for images_item_data in self.images:
            images_item = images_item_data.to_dict()
            images.append(images_item)

        description = self.description

        videos = []
        for videos_item_data in self.videos:
            videos_item = videos_item_data.to_dict()
            videos.append(videos_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "title": title,
                "url": url,
                "images": images,
                "description": description,
                "videos": videos,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.ogp_media import OgpMedia

        d = src_dict.copy()
        type = d.pop("type")

        title = d.pop("title")

        url = d.pop("url")

        images = []
        _images = d.pop("images")
        for images_item_data in _images:
            images_item = OgpMedia.from_dict(images_item_data)

            images.append(images_item)

        description = d.pop("description")

        videos = []
        _videos = d.pop("videos")
        for videos_item_data in _videos:
            videos_item = OgpMedia.from_dict(videos_item_data)

            videos.append(videos_item)

        ogp = cls(
            type=type,
            title=title,
            url=url,
            images=images,
            description=description,
            videos=videos,
        )

        ogp.additional_properties = d
        return ogp

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="TopicChangedEvent")


@_attrs_define
class TopicChangedEvent:
    """トピック変更イベント

    Attributes:
        user_id (str): 変更者UUID
        before (str): 変更前トピック
        after (str): 変更後トピック
    """

    user_id: str
    before: str
    after: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id

        before = self.before

        after = self.after

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "userId": user_id,
                "before": before,
                "after": after,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        user_id = d.pop("userId")

        before = d.pop("before")

        after = d.pop("after")

        topic_changed_event = cls(
            user_id=user_id,
            before=before,
            after=after,
        )

        topic_changed_event.additional_properties = d
        return topic_changed_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PostOAuth2Token")


@_attrs_define
class PostOAuth2Token:
    """
    Attributes:
        grant_type (str):
        code (Union[Unset, str]):
        redirect_uri (Union[Unset, str]):
        client_id (Union[Unset, str]):
        code_verifier (Union[Unset, str]):
        username (Union[Unset, str]):
        password (Union[Unset, str]):
        scope (Union[Unset, str]):
        refresh_token (Union[Unset, str]):
        client_secret (Union[Unset, str]):
    """

    grant_type: str
    code: Union[Unset, str] = UNSET
    redirect_uri: Union[Unset, str] = UNSET
    client_id: Union[Unset, str] = UNSET
    code_verifier: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    scope: Union[Unset, str] = UNSET
    refresh_token: Union[Unset, str] = UNSET
    client_secret: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        grant_type = self.grant_type

        code = self.code

        redirect_uri = self.redirect_uri

        client_id = self.client_id

        code_verifier = self.code_verifier

        username = self.username

        password = self.password

        scope = self.scope

        refresh_token = self.refresh_token

        client_secret = self.client_secret

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "grant_type": grant_type,
            }
        )
        if code is not UNSET:
            field_dict["code"] = code
        if redirect_uri is not UNSET:
            field_dict["redirect_uri"] = redirect_uri
        if client_id is not UNSET:
            field_dict["client_id"] = client_id
        if code_verifier is not UNSET:
            field_dict["code_verifier"] = code_verifier
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if scope is not UNSET:
            field_dict["scope"] = scope
        if refresh_token is not UNSET:
            field_dict["refresh_token"] = refresh_token
        if client_secret is not UNSET:
            field_dict["client_secret"] = client_secret

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        grant_type = d.pop("grant_type")

        code = d.pop("code", UNSET)

        redirect_uri = d.pop("redirect_uri", UNSET)

        client_id = d.pop("client_id", UNSET)

        code_verifier = d.pop("code_verifier", UNSET)

        username = d.pop("username", UNSET)

        password = d.pop("password", UNSET)

        scope = d.pop("scope", UNSET)

        refresh_token = d.pop("refresh_token", UNSET)

        client_secret = d.pop("client_secret", UNSET)

        post_o_auth_2_token = cls(
            grant_type=grant_type,
            code=code,
            redirect_uri=redirect_uri,
            client_id=client_id,
            code_verifier=code_verifier,
            username=username,
            password=password,
            scope=scope,
            refresh_token=refresh_token,
            client_secret=client_secret,
        )

        post_o_auth_2_token.additional_properties = d
        return post_o_auth_2_token

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.bot_mode import BotMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="PostBotRequest")


@_attrs_define
class PostBotRequest:
    """BOT作成リクエスト

    Attributes:
        name (str): BOTユーザーID
            自動的に接頭辞"BOT_"が付与されます
        display_name (str): BOTユーザー表示名
        description (str): BOTの説明
        mode (BotMode): BOT動作モード

            HTTP: HTTP Mode
            WebSocket: WebSocket Mode
        endpoint (Union[Unset, str]): BOTサーバーエンドポイント
            BOT動作モードがHTTPの場合必須です
    """

    name: str
    display_name: str
    description: str
    mode: BotMode
    endpoint: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        display_name = self.display_name

        description = self.description

        mode = self.mode.value

        endpoint = self.endpoint

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "displayName": display_name,
                "description": description,
                "mode": mode,
            }
        )
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        display_name = d.pop("displayName")

        description = d.pop("description")

        mode = BotMode(d.pop("mode"))

        endpoint = d.pop("endpoint", UNSET)

        post_bot_request = cls(
            name=name,
            display_name=display_name,
            description=description,
            mode=mode,
            endpoint=endpoint,
        )

        post_bot_request.additional_properties = d
        return post_bot_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.user_account_state import UserAccountState
from ..models.user_permission import UserPermission

if TYPE_CHECKING:
    from ..models.user_tag import UserTag


T = TypeVar("T", bound="OIDCTraqUserInfo")


@_attrs_define
class OIDCTraqUserInfo:
    """traQ特有のユーザー詳細情報

    Attributes:
        bio (str): 自己紹介(biography)
        groups (List[str]): 所属グループのUUIDの配列
        tags (List['UserTag']): タグリスト
        last_online (Union[None, datetime.datetime]): 最終オンライン日時
        twitter_id (str): Twitter ID
        display_name (str): ユーザー表示名
        icon_file_id (str): アイコンファイルUUID
        bot (bool): BOTかどうか
        state (UserAccountState): ユーザーアカウント状態
            0: 停止
            1: 有効
            2: 一時停止
        permissions (List[UserPermission]): 所有している権限の配列
        home_channel (Union[None, str]): ホームチャンネル
    """

    bio: str
    groups: List[str]
    tags: List["UserTag"]
    last_online: Union[None, datetime.datetime]
    twitter_id: str
    display_name: str
    icon_file_id: str
    bot: bool
    state: UserAccountState
    permissions: List[UserPermission]
    home_channel: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bio = self.bio

        groups = self.groups

        tags = []
        for tags_item_data in self.tags:
            tags_item = tags_item_data.to_dict()
            tags.append(tags_item)

        last_online: Union[None, str]
        if isinstance(self.last_online, datetime.datetime):
            last_online = self.last_online.isoformat()
        else:
            last_online = self.last_online

        twitter_id = self.twitter_id

        display_name = self.display_name

        icon_file_id = self.icon_file_id

        bot = self.bot

        state = self.state.value

        permissions = []
        for permissions_item_data in self.permissions:
            permissions_item = permissions_item_data.value
            permissions.append(permissions_item)

        home_channel: Union[None, str]
        home_channel = self.home_channel

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "bio": bio,
                "groups": groups,
                "tags": tags,
                "last_online": last_online,
                "twitter_id": twitter_id,
                "display_name": display_name,
                "icon_file_id": icon_file_id,
                "bot": bot,
                "state": state,
                "permissions": permissions,
                "home_channel": home_channel,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_tag import UserTag

        d = src_dict.copy()
        bio = d.pop("bio")

        groups = cast(List[str], d.pop("groups"))

        tags = []
        _tags = d.pop("tags")
        for tags_item_data in _tags:
            tags_item = UserTag.from_dict(tags_item_data)

            tags.append(tags_item)

        def _parse_last_online(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_online_type_0 = isoparse(data)

                return last_online_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        last_online = _parse_last_online(d.pop("last_online"))

        twitter_id = d.pop("twitter_id")

        display_name = d.pop("display_name")

        icon_file_id = d.pop("icon_file_id")

        bot = d.pop("bot")

        state = UserAccountState(d.pop("state"))

        permissions = []
        _permissions = d.pop("permissions")
        for permissions_item_data in _permissions:
            permissions_item = UserPermission(permissions_item_data)

            permissions.append(permissions_item)

        def _parse_home_channel(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        home_channel = _parse_home_channel(d.pop("home_channel"))

        oidc_traq_user_info = cls(
            bio=bio,
            groups=groups,
            tags=tags,
            last_online=last_online,
            twitter_id=twitter_id,
            display_name=display_name,
            icon_file_id=icon_file_id,
            bot=bot,
            state=state,
            permissions=permissions,
            home_channel=home_channel,
        )

        oidc_traq_user_info.additional_properties = d
        return oidc_traq_user_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="Channel")


@_attrs_define
class Channel:
    """チャンネル

    Attributes:
        id (str): チャンネルUUID
        parent_id (Union[None, str]): 親チャンネルUUID
        archived (bool): チャンネルがアーカイブされているかどうか
        force (bool): 強制通知チャンネルかどうか
        topic (str): チャンネルトピック
        name (str): チャンネル名
        children (List[str]): 子チャンネルのUUID配列
    """

    id: str
    parent_id: Union[None, str]
    archived: bool
    force: bool
    topic: str
    name: str
    children: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        parent_id: Union[None, str]
        parent_id = self.parent_id

        archived = self.archived

        force = self.force

        topic = self.topic

        name = self.name

        children = self.children

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "parentId": parent_id,
                "archived": archived,
                "force": force,
                "topic": topic,
                "name": name,
                "children": children,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        def _parse_parent_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        parent_id = _parse_parent_id(d.pop("parentId"))

        archived = d.pop("archived")

        force = d.pop("force")

        topic = d.pop("topic")

        name = d.pop("name")

        children = cast(List[str], d.pop("children"))

        channel = cls(
            id=id,
            parent_id=parent_id,
            archived=archived,
            force=force,
            topic=topic,
            name=name,
            children=children,
        )

        channel.additional_properties = d
        return channel

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.channel import Channel
    from ..models.dm_channel import DMChannel


T = TypeVar("T", bound="ChannelList")


@_attrs_define
class ChannelList:
    """GET /channelsレスポンス

    Attributes:
        public (List['Channel']): パブリックチャンネルの配列
        dm (Union[Unset, List['DMChannel']]): ダイレクトメッセージチャンネルの配列
    """

    public: List["Channel"]
    dm: Union[Unset, List["DMChannel"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        public = []
        for public_item_data in self.public:
            public_item = public_item_data.to_dict()
            public.append(public_item)

        dm: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.dm, Unset):
            dm = []
            for dm_item_data in self.dm:
                dm_item = dm_item_data.to_dict()
                dm.append(dm_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "public": public,
            }
        )
        if dm is not UNSET:
            field_dict["dm"] = dm

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.channel import Channel
        from ..models.dm_channel import DMChannel

        d = src_dict.copy()
        public = []
        _public = d.pop("public")
        for public_item_data in _public:
            public_item = Channel.from_dict(public_item_data)

            public.append(public_item)

        dm = []
        _dm = d.pop("dm", UNSET)
        for dm_item_data in _dm or []:
            dm_item = DMChannel.from_dict(dm_item_data)

            dm.append(dm_item)

        channel_list = cls(
            public=public,
            dm=dm,
        )

        channel_list.additional_properties = d
        return channel_list

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.file_info_thumbnail_type_0 import FileInfoThumbnailType0
    from ..models.thumbnail_info import ThumbnailInfo


T = TypeVar("T", bound="FileInfo")


@_attrs_define
class FileInfo:
    """ファイル情報

    Attributes:
        id (str): ファイルUUID
        name (str): ファイル名
        mime (str): MIMEタイプ
        size (int): ファイルサイズ
        md5 (str): MD5ハッシュ
        is_animated_image (bool): アニメーション画像かどうか
        created_at (datetime.datetime): アップロード日時
        thumbnails (List['ThumbnailInfo']):
        thumbnail (Union['FileInfoThumbnailType0', None]): サムネイル情報
            サムネイルが存在しない場合はnullになります
            Deprecated: thumbnailsを参照してください
        channel_id (Union[None, str]): 属しているチャンネルUUID
        uploader_id (Union[None, str]): アップロード者UUID
    """

    id: str
    name: str
    mime: str
    size: int
    md5: str
    is_animated_image: bool
    created_at: datetime.datetime
    thumbnails: List["ThumbnailInfo"]
    thumbnail: Union["FileInfoThumbnailType0", None]
    channel_id: Union[None, str]
    uploader_id: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.file_info_thumbnail_type_0 import FileInfoThumbnailType0

        id = self.id

        name = self.name

        mime = self.mime

        size = self.size

        md5 = self.md5

        is_animated_image = self.is_animated_image

        created_at = self.created_at.isoformat()

        thumbnails = []
        for thumbnails_item_data in self.thumbnails:
            thumbnails_item = thumbnails_item_data.to_dict()
            thumbnails.append(thumbnails_item)

        thumbnail: Union[Dict[str, Any], None]
        if isinstance(self.thumbnail, FileInfoThumbnailType0):
            thumbnail = self.thumbnail.to_dict()
        else:
            thumbnail = self.thumbnail

        channel_id: Union[None, str]
        channel_id = self.channel_id

        uploader_id: Union[None, str]
        uploader_id = self.uploader_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "mime": mime,
                "size": size,
                "md5": md5,
                "isAnimatedImage": is_animated_image,
                "createdAt": created_at,
                "thumbnails": thumbnails,
                "thumbnail": thumbnail,
                "channelId": channel_id,
                "uploaderId": uploader_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.file_info_thumbnail_type_0 import FileInfoThumbnailType0
        from ..models.thumbnail_info import ThumbnailInfo

        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        mime = d.pop("mime")

        size = d.pop("size")

        md5 = d.pop("md5")

        is_animated_image = d.pop("isAnimatedImage")

        created_at = isoparse(d.pop("createdAt"))

        thumbnails = []
        _thumbnails = d.pop("thumbnails")
        for thumbnails_item_data in _thumbnails:
            thumbnails_item = ThumbnailInfo.from_dict(thumbnails_item_data)

            thumbnails.append(thumbnails_item)

        def _parse_thumbnail(data: object) -> Union["FileInfoThumbnailType0", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                thumbnail_type_0 = FileInfoThumbnailType0.from_dict(data)

                return thumbnail_type_0
            except:  # noqa: E722
                pass
            return cast(Union["FileInfoThumbnailType0", None], data)

        thumbnail = _parse_thumbnail(d.pop("thumbnail"))

        def _parse_channel_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        channel_id = _parse_channel_id(d.pop("channelId"))

        def _parse_uploader_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        uploader_id = _parse_uploader_id(d.pop("uploaderId"))

        file_info = cls(
            id=id,
            name=name,
            mime=mime,
            size=size,
            md5=md5,
            is_animated_image=is_animated_image,
            created_at=created_at,
            thumbnails=thumbnails,
            thumbnail=thumbnail,
            channel_id=channel_id,
            uploader_id=uploader_id,
        )

        file_info.additional_properties = d
        return file_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.message import Message


T = TypeVar("T", bound="ClippedMessage")


@_attrs_define
class ClippedMessage:
    """クリップされたメッセージ

    Attributes:
        message (Message): メッセージ
        clipped_at (datetime.datetime): クリップした日時
    """

    message: "Message"
    clipped_at: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message.to_dict()

        clipped_at = self.clipped_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
                "clippedAt": clipped_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message import Message

        d = src_dict.copy()
        message = Message.from_dict(d.pop("message"))

        clipped_at = isoparse(d.pop("clippedAt"))

        clipped_message = cls(
            message=message,
            clipped_at=clipped_at,
        )

        clipped_message.additional_properties = d
        return clipped_message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union, cast

import httpx
from dateutil.parser import isoparse

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.file_info import FileInfo
from ...models.get_files_order import GetFilesOrder
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    channel_id: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = 0,
    since: Union[Unset, datetime.datetime] = isoparse("0001-01-01T00:00:00.000000Z"),
    until: Union[Unset, datetime.datetime] = UNSET,
    inclusive: Union[Unset, bool] = False,
    order: Union[Unset, GetFilesOrder] = GetFilesOrder.DESC,
    mine: Union[Unset, bool] = False,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["channelId"] = channel_id

    params["limit"] = limit

    params["offset"] = offset

    json_since: Union[Unset, str] = UNSET
    if not isinstance(since, Unset):
        json_since = since.isoformat()
    params["since"] = json_since

    json_until: Union[Unset, str] = UNSET
    if not isinstance(until, Unset):
        json_until = until.isoformat()
    params["until"] = json_until

    params["inclusive"] = inclusive

    json_order: Union[Unset, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value

    params["order"] = json_order

    params["mine"] = mine

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/files",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, List["FileInfo"]]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = FileInfo.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = cast(Any, None)
        return response_400
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, List["FileInfo"]]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    channel_id: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = 0,
    since: Union[Unset, datetime.datetime] = isoparse("0001-01-01T00:00:00.000000Z"),
    until: Union[Unset, datetime.datetime] = UNSET,
    inclusive: Union[Unset, bool] = False,
    order: Union[Unset, GetFilesOrder] = GetFilesOrder.DESC,
    mine: Union[Unset, bool] = False,
) -> Response[Union[Any, List["FileInfo"]]]:
    """ファイルメタのリストを取得

     指定したクエリでファイルメタのリストを取得します。
    クエリパラメータ`channelId`, `mine`の少なくともいずれかが必須です。

    Args:
        channel_id (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):  Default: 0.
        since (Union[Unset, datetime.datetime]):  Default:
            isoparse('0001-01-01T00:00:00.000000Z').
        until (Union[Unset, datetime.datetime]):
        inclusive (Union[Unset, bool]):  Default: False.
        order (Union[Unset, GetFilesOrder]):  Default: GetFilesOrder.DESC.
        mine (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['FileInfo']]]
    """

    kwargs = _get_kwargs(
        channel_id=channel_id,
        limit=limit,
        offset=offset,
        since=since,
        until=until,
        inclusive=inclusive,
        order=order,
        mine=mine,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    channel_id: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = 0,
    since: Union[Unset, datetime.datetime] = isoparse("0001-01-01T00:00:00.000000Z"),
    until: Union[Unset, datetime.datetime] = UNSET,
    inclusive: Union[Unset, bool] = False,
    order: Union[Unset, GetFilesOrder] = GetFilesOrder.DESC,
    mine: Union[Unset, bool] = False,
) -> Optional[Union[Any, List["FileInfo"]]]:
    """ファイルメタのリストを取得

     指定したクエリでファイルメタのリストを取得します。
    クエリパラメータ`channelId`, `mine`の少なくともいずれかが必須です。

    Args:
        channel_id (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):  Default: 0.
        since (Union[Unset, datetime.datetime]):  Default:
            isoparse('0001-01-01T00:00:00.000000Z').
        until (Union[Unset, datetime.datetime]):
        inclusive (Union[Unset, bool]):  Default: False.
        order (Union[Unset, GetFilesOrder]):  Default: GetFilesOrder.DESC.
        mine (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['FileInfo']]
    """

    return sync_detailed(
        client=client,
        channel_id=channel_id,
        limit=limit,
        offset=offset,
        since=since,
        until=until,
        inclusive=inclusive,
        order=order,
        mine=mine,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    channel_id: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = 0,
    since: Union[Unset, datetime.datetime] = isoparse("0001-01-01T00:00:00.000000Z"),
    until: Union[Unset, datetime.datetime] = UNSET,
    inclusive: Union[Unset, bool] = False,
    order: Union[Unset, GetFilesOrder] = GetFilesOrder.DESC,
    mine: Union[Unset, bool] = False,
) -> Response[Union[Any, List["FileInfo"]]]:
    """ファイルメタのリストを取得

     指定したクエリでファイルメタのリストを取得します。
    クエリパラメータ`channelId`, `mine`の少なくともいずれかが必須です。

    Args:
        channel_id (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):  Default: 0.
        since (Union[Unset, datetime.datetime]):  Default:
            isoparse('0001-01-01T00:00:00.000000Z').
        until (Union[Unset, datetime.datetime]):
        inclusive (Union[Unset, bool]):  Default: False.
        order (Union[Unset, GetFilesOrder]):  Default: GetFilesOrder.DESC.
        mine (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, List['FileInfo']]]
    """

    kwargs = _get_kwargs(
        channel_id=channel_id,
        limit=limit,
        offset=offset,
        since=since,
        until=until,
        inclusive=inclusive,
        order=order,
        mine=mine,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    channel_id: Union[Unset, str] = UNSET,
    limit: Union[Unset, int] = UNSET,
    offset: Union[Unset, int] = 0,
    since: Union[Unset, datetime.datetime] = isoparse("0001-01-01T00:00:00.000000Z"),
    until: Union[Unset, datetime.datetime] = UNSET,
    inclusive: Union[Unset, bool] = False,
    order: Union[Unset, GetFilesOrder] = GetFilesOrder.DESC,
    mine: Union[Unset, bool] = False,
) -> Optional[Union[Any, List["FileInfo"]]]:
    """ファイルメタのリストを取得

     指定したクエリでファイルメタのリストを取得します。
    クエリパラメータ`channelId`, `mine`の少なくともいずれかが必須です。

    Args:
        channel_id (Union[Unset, str]):
        limit (Union[Unset, int]):
        offset (Union[Unset, int]):  Default: 0.
        since (Union[Unset, datetime.datetime]):  Default:
            isoparse('0001-01-01T00:00:00.000000Z').
        until (Union[Unset, datetime.datetime]):
        inclusive (Union[Unset, bool]):  Default: False.
        order (Union[Unset, GetFilesOrder]):  Default: GetFilesOrder.DESC.
        mine (Union[Unset, bool]):  Default: False.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, List['FileInfo']]
    """

    return (
        await asyncio_detailed(
            client=client,
            channel_id=channel_id,
            limit=limit,
            offset=offset,
            since=since,
            until=until,
            inclusive=inclusive,
            order=order,
            mine=mine,
        )
    ).parsed



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.oidc_traq_user_info import OIDCTraqUserInfo


T = TypeVar("T", bound="OIDCUserInfo")


@_attrs_define
class OIDCUserInfo:
    """自分のユーザー詳細情報

    Attributes:
        sub (str): ユーザーUUID
        name (str): ユーザー名
        preferred_username (str): ユーザー名
        picture (str): アイコン画像URL
        updated_at (Union[Unset, int]): 更新日時
        traq (Union[Unset, OIDCTraqUserInfo]): traQ特有のユーザー詳細情報
    """

    sub: str
    name: str
    preferred_username: str
    picture: str
    updated_at: Union[Unset, int] = UNSET
    traq: Union[Unset, "OIDCTraqUserInfo"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sub = self.sub

        name = self.name

        preferred_username = self.preferred_username

        picture = self.picture

        updated_at = self.updated_at

        traq: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.traq, Unset):
            traq = self.traq.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sub": sub,
                "name": name,
                "preferred_username": preferred_username,
                "picture": picture,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if traq is not UNSET:
            field_dict["traq"] = traq

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.oidc_traq_user_info import OIDCTraqUserInfo

        d = src_dict.copy()
        sub = d.pop("sub")

        name = d.pop("name")

        preferred_username = d.pop("preferred_username")

        picture = d.pop("picture")

        updated_at = d.pop("updated_at", UNSET)

        _traq = d.pop("traq", UNSET)
        traq: Union[Unset, OIDCTraqUserInfo]
        if isinstance(_traq, Unset):
            traq = UNSET
        else:
            traq = OIDCTraqUserInfo.from_dict(_traq)

        oidc_user_info = cls(
            sub=sub,
            name=name,
            preferred_username=preferred_username,
            picture=picture,
            updated_at=updated_at,
            traq=traq,
        )

        oidc_user_info.additional_properties = d
        return oidc_user_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .active_o_auth_2_token import ActiveOAuth2Token
from .activity_timeline_message import ActivityTimelineMessage
from .bot import Bot
from .bot_detail import BotDetail
from .bot_event_log import BotEventLog
from .bot_event_result import BotEventResult
from .bot_mode import BotMode
from .bot_state import BotState
from .bot_tokens import BotTokens
from .bot_user import BotUser
from .change_stamp_image_body import ChangeStampImageBody
from .channel import Channel
from .channel_event import ChannelEvent
from .channel_event_type import ChannelEventType
from .channel_list import ChannelList
from .channel_stats import ChannelStats
from .channel_stats_stamp import ChannelStatsStamp
from .channel_stats_user import ChannelStatsUser
from .channel_subscribe_level import ChannelSubscribeLevel
from .channel_topic import ChannelTopic
from .channel_view_state import ChannelViewState
from .channel_viewer import ChannelViewer
from .child_created_event import ChildCreatedEvent
from .clip_folder import ClipFolder
from .clipped_message import ClippedMessage
from .dm_channel import DMChannel
from .external_provider_user import ExternalProviderUser
from .file_info import FileInfo
from .file_info_thumbnail_type_0 import FileInfoThumbnailType0
from .forced_notification_changed_event import ForcedNotificationChangedEvent
from .get_channel_events_order import GetChannelEventsOrder
from .get_clips_order import GetClipsOrder
from .get_direct_messages_order import GetDirectMessagesOrder
from .get_files_order import GetFilesOrder
from .get_messages_order import GetMessagesOrder
from .get_notify_citation import GetNotifyCitation
from .get_stamps_type import GetStampsType
from .get_webhook_messages_order import GetWebhookMessagesOrder
from .login_session import LoginSession
from .message import Message
from .message_clip import MessageClip
from .message_pin import MessagePin
from .message_stamp import MessageStamp
from .my_channel_view_state import MyChannelViewState
from .my_user_detail import MyUserDetail
from .name_changed_event import NameChangedEvent
from .o_auth_2_authorization import OAuth2Authorization
from .o_auth_2_client import OAuth2Client
from .o_auth_2_client_detail import OAuth2ClientDetail
from .o_auth_2_decide import OAuth2Decide
from .o_auth_2_prompt import OAuth2Prompt
from .o_auth_2_response_type import OAuth2ResponseType
from .o_auth_2_revoke import OAuth2Revoke
from .o_auth_2_scope import OAuth2Scope
from .o_auth_2_token import OAuth2Token
from .ogp import Ogp
from .ogp_media import OgpMedia
from .oidc_traq_user_info import OIDCTraqUserInfo
from .oidc_user_info import OIDCUserInfo
from .parent_changed_event import ParentChangedEvent
from .patch_bot_request import PatchBotRequest
from .patch_channel_request import PatchChannelRequest
from .patch_channel_subscribers_request import PatchChannelSubscribersRequest
from .patch_client_request import PatchClientRequest
from .patch_clip_folder_request import PatchClipFolderRequest
from .patch_group_member_request import PatchGroupMemberRequest
from .patch_me_request import PatchMeRequest
from .patch_stamp_palette_request import PatchStampPaletteRequest
from .patch_stamp_request import PatchStampRequest
from .patch_user_group_request import PatchUserGroupRequest
from .patch_user_request import PatchUserRequest
from .patch_user_tag_request import PatchUserTagRequest
from .patch_webhook_request import PatchWebhookRequest
from .pin import Pin
from .pin_added_event import PinAddedEvent
from .pin_removed_event import PinRemovedEvent
from .post_bot_action_join_request import PostBotActionJoinRequest
from .post_bot_action_leave_request import PostBotActionLeaveRequest
from .post_bot_request import PostBotRequest
from .post_channel_request import PostChannelRequest
from .post_client_request import PostClientRequest
from .post_clip_folder_message_request import PostClipFolderMessageRequest
from .post_clip_folder_request import PostClipFolderRequest
from .post_file_request import PostFileRequest
from .post_link_external_account import PostLinkExternalAccount
from .post_login_request import PostLoginRequest
from .post_message_request import PostMessageRequest
from .post_message_stamp_request import PostMessageStampRequest
from .post_my_fcm_device_request import PostMyFCMDeviceRequest
from .post_o_auth_2_token import PostOAuth2Token
from .post_stamp_palette_request import PostStampPaletteRequest
from .post_stamp_request import PostStampRequest
from .post_star_request import PostStarRequest
from .post_unlink_external_account import PostUnlinkExternalAccount
from .post_user_group_admin import PostUserGroupAdmin
from .post_user_group_request import PostUserGroupRequest
from .post_user_request import PostUserRequest
from .post_user_tag_request import PostUserTagRequest
from .post_web_rtc_authenticate_request import PostWebRTCAuthenticateRequest
from .post_webhook_request import PostWebhookRequest
from .put_channel_subscribe_level_request import PutChannelSubscribeLevelRequest
from .put_channel_subscribers_request import PutChannelSubscribersRequest
from .put_channel_topic_request import PutChannelTopicRequest
from .put_my_password_request import PutMyPasswordRequest
from .put_notify_citation_request import PutNotifyCitationRequest
from .put_user_icon_request import PutUserIconRequest
from .put_user_password_request import PutUserPasswordRequest
from .search_messages_message_search_result import SearchMessagesMessageSearchResult
from .search_messages_sort import SearchMessagesSort
from .stamp import Stamp
from .stamp_history_entry import StampHistoryEntry
from .stamp_palette import StampPalette
from .stamp_stats import StampStats
from .stamp_with_thumbnail import StampWithThumbnail
from .subscribers_changed_event import SubscribersChangedEvent
from .tag import Tag
from .thumbnail_info import ThumbnailInfo
from .thumbnail_type import ThumbnailType
from .topic_changed_event import TopicChangedEvent
from .unread_channel import UnreadChannel
from .user import User
from .user_account_state import UserAccountState
from .user_detail import UserDetail
from .user_group import UserGroup
from .user_group_member import UserGroupMember
from .user_permission import UserPermission
from .user_settings import UserSettings
from .user_stats import UserStats
from .user_stats_stamp import UserStatsStamp
from .user_subscribe_state import UserSubscribeState
from .user_tag import UserTag
from .version import Version
from .version_flags import VersionFlags
from .visibility_changed_event import VisibilityChangedEvent
from .web_rtc_authenticate_result import WebRTCAuthenticateResult
from .web_rtc_user_state import WebRTCUserState
from .web_rtc_user_state_sessions_item import WebRTCUserStateSessionsItem
from .webhook import Webhook

__all__ = (
    "ActiveOAuth2Token",
    "ActivityTimelineMessage",
    "Bot",
    "BotDetail",
    "BotEventLog",
    "BotEventResult",
    "BotMode",
    "BotState",
    "BotTokens",
    "BotUser",
    "ChangeStampImageBody",
    "Channel",
    "ChannelEvent",
    "ChannelEventType",
    "ChannelList",
    "ChannelStats",
    "ChannelStatsStamp",
    "ChannelStatsUser",
    "ChannelSubscribeLevel",
    "ChannelTopic",
    "ChannelViewer",
    "ChannelViewState",
    "ChildCreatedEvent",
    "ClipFolder",
    "ClippedMessage",
    "DMChannel",
    "ExternalProviderUser",
    "FileInfo",
    "FileInfoThumbnailType0",
    "ForcedNotificationChangedEvent",
    "GetChannelEventsOrder",
    "GetClipsOrder",
    "GetDirectMessagesOrder",
    "GetFilesOrder",
    "GetMessagesOrder",
    "GetNotifyCitation",
    "GetStampsType",
    "GetWebhookMessagesOrder",
    "LoginSession",
    "Message",
    "MessageClip",
    "MessagePin",
    "MessageStamp",
    "MyChannelViewState",
    "MyUserDetail",
    "NameChangedEvent",
    "OAuth2Authorization",
    "OAuth2Client",
    "OAuth2ClientDetail",
    "OAuth2Decide",
    "OAuth2Prompt",
    "OAuth2ResponseType",
    "OAuth2Revoke",
    "OAuth2Scope",
    "OAuth2Token",
    "Ogp",
    "OgpMedia",
    "OIDCTraqUserInfo",
    "OIDCUserInfo",
    "ParentChangedEvent",
    "PatchBotRequest",
    "PatchChannelRequest",
    "PatchChannelSubscribersRequest",
    "PatchClientRequest",
    "PatchClipFolderRequest",
    "PatchGroupMemberRequest",
    "PatchMeRequest",
    "PatchStampPaletteRequest",
    "PatchStampRequest",
    "PatchUserGroupRequest",
    "PatchUserRequest",
    "PatchUserTagRequest",
    "PatchWebhookRequest",
    "Pin",
    "PinAddedEvent",
    "PinRemovedEvent",
    "PostBotActionJoinRequest",
    "PostBotActionLeaveRequest",
    "PostBotRequest",
    "PostChannelRequest",
    "PostClientRequest",
    "PostClipFolderMessageRequest",
    "PostClipFolderRequest",
    "PostFileRequest",
    "PostLinkExternalAccount",
    "PostLoginRequest",
    "PostMessageRequest",
    "PostMessageStampRequest",
    "PostMyFCMDeviceRequest",
    "PostOAuth2Token",
    "PostStampPaletteRequest",
    "PostStampRequest",
    "PostStarRequest",
    "PostUnlinkExternalAccount",
    "PostUserGroupAdmin",
    "PostUserGroupRequest",
    "PostUserRequest",
    "PostUserTagRequest",
    "PostWebhookRequest",
    "PostWebRTCAuthenticateRequest",
    "PutChannelSubscribeLevelRequest",
    "PutChannelSubscribersRequest",
    "PutChannelTopicRequest",
    "PutMyPasswordRequest",
    "PutNotifyCitationRequest",
    "PutUserIconRequest",
    "PutUserPasswordRequest",
    "SearchMessagesMessageSearchResult",
    "SearchMessagesSort",
    "Stamp",
    "StampHistoryEntry",
    "StampPalette",
    "StampStats",
    "StampWithThumbnail",
    "SubscribersChangedEvent",
    "Tag",
    "ThumbnailInfo",
    "ThumbnailType",
    "TopicChangedEvent",
    "UnreadChannel",
    "User",
    "UserAccountState",
    "UserDetail",
    "UserGroup",
    "UserGroupMember",
    "UserPermission",
    "UserSettings",
    "UserStats",
    "UserStatsStamp",
    "UserSubscribeState",
    "UserTag",
    "Version",
    "VersionFlags",
    "VisibilityChangedEvent",
    "Webhook",
    "WebRTCAuthenticateResult",
    "WebRTCUserState",
    "WebRTCUserStateSessionsItem",
)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.message_stamp import MessageStamp


T = TypeVar("T", bound="Message")


@_attrs_define
class Message:
    """メッセージ

    Attributes:
        id (str): メッセージUUID
        user_id (str): 投稿者UUID
        channel_id (str): チャンネルUUID
        content (str): メッセージ本文
        created_at (datetime.datetime): 投稿日時
        updated_at (datetime.datetime): 編集日時
        pinned (bool): ピン留めされているかどうか
        stamps (List['MessageStamp']): 押されているスタンプの配列
        thread_id (Union[None, str]): スレッドUUID
    """

    id: str
    user_id: str
    channel_id: str
    content: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    pinned: bool
    stamps: List["MessageStamp"]
    thread_id: Union[None, str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        user_id = self.user_id

        channel_id = self.channel_id

        content = self.content

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        pinned = self.pinned

        stamps = []
        for stamps_item_data in self.stamps:
            stamps_item = stamps_item_data.to_dict()
            stamps.append(stamps_item)

        thread_id: Union[None, str]
        thread_id = self.thread_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "userId": user_id,
                "channelId": channel_id,
                "content": content,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "pinned": pinned,
                "stamps": stamps,
                "threadId": thread_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.message_stamp import MessageStamp

        d = src_dict.copy()
        id = d.pop("id")

        user_id = d.pop("userId")

        channel_id = d.pop("channelId")

        content = d.pop("content")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        pinned = d.pop("pinned")

        stamps = []
        _stamps = d.pop("stamps")
        for stamps_item_data in _stamps:
            stamps_item = MessageStamp.from_dict(stamps_item_data)

            stamps.append(stamps_item)

        def _parse_thread_id(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        thread_id = _parse_thread_id(d.pop("threadId"))

        message = cls(
            id=id,
            user_id=user_id,
            channel_id=channel_id,
            content=content,
            created_at=created_at,
            updated_at=updated_at,
            pinned=pinned,
            stamps=stamps,
            thread_id=thread_id,
        )

        message.additional_properties = d
        return message

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="StampHistoryEntry")


@_attrs_define
class StampHistoryEntry:
    """スタンプ履歴の1項目

    Attributes:
        stamp_id (str): スタンプUUID
        datetime_ (datetime.datetime): 使用日時
    """

    stamp_id: str
    datetime_: datetime.datetime
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        stamp_id = self.stamp_id

        datetime_ = self.datetime_.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "stampId": stamp_id,
                "datetime": datetime_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        stamp_id = d.pop("stampId")

        datetime_ = isoparse(d.pop("datetime"))

        stamp_history_entry = cls(
            stamp_id=stamp_id,
            datetime_=datetime_,
        )

        stamp_history_entry.additional_properties = d
        return stamp_history_entry

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
